/*
** $Log:   C:/shell/vcs/chmod.c_v  $
**
**   Rev 3.5   01 Oct 1986 13:24:14   kent
**Put all globals into shell.h
**
**   Rev 3.4   01 Oct 1986  9:22:32   kent
**Initial revision.
*/

/* settings of the st_attr field */
#define ST_RDONLY	0x01	/* read only file */
#define ST_HIDDEN	0x02	/* hidden file */
#define ST_SYSTEM	0x04	/* system file */
#define ST_VLABEL	0x08	/* volume label */
#define ST_DIRECT	0x10	/* file is a sub-directory */
#define ST_ARCHIV	0x20	/* set when file has been written and closed */

#include "shell.h"
void print_error();
int
#ifdef MAIN
main
#else
ch_mod
#endif
(argc,argv)
	char *argv[];
{
	int or_mask,and_mask,file_stat;
	struct registers regs;
	char *current;
#ifdef AZTEC86
	extern unsigned _dsval;
	regs.ds = (int)_dsval;
#endif
#ifdef MSC
	GETSEGS(&regs);
#endif
	if (argc==1)
	{
		fprintf(stderr,"Usage : chmod +|-[ahrsw] file [file ...]\n");
		return -1;
	}
	/* set attributes to default */
	or_mask = 0; and_mask = 0xFFFF;
	while(--argc)
	{
		current = *(++argv);
		switch (*current)
		{
		case '-':
			while (*++current)
			{
				switch(*current)
				{
				case 'w':
				case 'W':
					or_mask |= ST_RDONLY;
					break;
				case 'h':
				case 'H':
					and_mask &= (ST_HIDDEN ^ 0xFFFF);
					break;
				case 'r':
				case 'R':
					or_mask |= ST_HIDDEN;
					break;
				case 'a':
				case 'A':
					and_mask &= (ST_ARCHIV ^ 0xFFFF);
					break;
				case 's':
				case 'S':
					and_mask &= (ST_SYSTEM ^ 0xFFFF);
					break;
				default:
					write(2,"invalid attribute\r\n",19);
					return -1;
				}
			}
			break;
		case '+':
			while(*++current)
			{
				switch(*current)
				{
				case 'w':
				case 'W':
					and_mask &= (ST_RDONLY ^ 0xFFFF);
					break;
				case 'h':
				case 'H':
					or_mask |= ST_HIDDEN;
					break;
				case 's':
				case 'S':
					or_mask |= ST_SYSTEM;
					break;
				case 'r':
				case 'R':
					and_mask &= (ST_HIDDEN ^ 0xFFFF);
					break;
				case 'a':
				case 'A':
					or_mask |= ST_ARCHIV;
					break;
				default:
					write(2,"invalid attribute\r\n",19);
					return -1;

				}
			}
			break;
		default:
			/* get current attribute */
			regs.ax = 0x4300;
			regs.dx = (int)current;
#ifdef AZTEC86
			if (sysint21(&regs,&regs) & 1)
#endif
#ifdef MSC
			sysint21(&regs,&regs);
			if (regs.cflag)
#endif
			{
				print_error("chmod",regs.ax);
				continue;
			}
			file_stat = regs.cx;
			fprintf(stderr,"current attribute for %s = %x\n",
				current,file_stat);
			/* set new attribute */
			file_stat |= or_mask;
			file_stat &= and_mask;
			regs.ax = 0x4301;
			regs.dx = (int)current;
			regs.cx = file_stat;
#ifdef AZTEC86
			if (sysint21(&regs,&regs) & 1)
#endif
#ifdef MSC
			sysint21(&regs,&regs);
			if (regs.cflag)
#endif
			{
				print_error("chmod",regs.ax);
				continue;
			}
			/* get attribute to see if it changed */
			regs.ax = 0x4300;
			regs.dx = (int)current;
			(void)sysint21(&regs,&regs);
			file_stat = regs.cx;
			fprintf(stderr,"new attribute for %s = %x\n",
				current,file_stat);
			break;
		}
	}
	return 0;
}
#ifdef MAIN

#ifndef NULL
#define NULL ((void *)0)
#endif
static char *errors[] =
{
NULL,
"bad function",
"file not found",
"path not found",
"too many open files",
"access denied",
"invalid handle",
"memory trashed",
"insufficient memory",
"invalid memory block address",
"invalid environment",
"invalid format",
"invalid access code",
"invalid data",
"mysterious internal DOS error",
"invalid drive",
"can't remove current directory",
"not same device",
"no more files",
"unknown error"
};

void print_error(string,code)
	char *string;
{
	write(2,string,strlen(string));
	write(2," : ",3);
	if (code < 0 || code > 18)
		code = 19;
	write(2,errors[code],strlen(errors[code]));
	write(2,"\r\n",2);
}
#endif
