/*
** $Log:   C:/shell/vcs/mvcp.c_v  $
**
**   Rev 3.5   02 Oct 1986 13:12:16   kent
**lint pass over everything
**
**   Rev 3.4   01 Oct 1986  9:25:12   kent
**Initial revision.
*/
/*lint -e713
 * mvcp.c - routines common to mv and cp
 */
#include "shell.h"
#ifdef MSC
#include <sys\types.h>
#include <sys\stat.h>
#endif
#define ERROR 0xFFFF

typedef unsigned int paragraph;

extern int errno;

extern char *me;

/*
** get_big_mem grabs a memory block outside our current allocation, so
** that the shells heap doesnt get enlarged by grabbing a huge buffer.
** This is important since we have no way to shrink the heap. It can
** reliably allocate up to 64K - 16 bytes, so 63K is a safe top end.
** beyond that, I would have to go to longs to specify sizes, and thats messy.
*/
paragraph get_big_mem(size,avail)
	unsigned size;
	unsigned *avail;
{
	struct registers regs;
	if (size % 16 != 0)	/* not an even paragraph amount */
		size += 15;
	size >>= 4;			/* shift to a paragraph count */
	regs.ax = 0x4800;	/* allocate memory call */
	regs.bx = size;		/* parm */
#ifdef AZTEC86
	if (sysint21(&regs,&regs) & 1)	/* error returned from sysint */
#endif
#ifdef MSC
	sysint21(&regs,&regs);
	if (regs.cflag)
#endif
	{
		/* if carry is set, the alloc failed, and bx contains the available
		** number of paragraphs
		*/
		*avail = ((unsigned)regs.bx) << 4;	/* shift to byte count */
		return 0;
	}
	*avail = 0;
	return (paragraph)regs.ax;	/* paragraph of allocated block */
}

/*
** converse of above
*/
int free_big_mem(base)
	paragraph base;
{
	struct registers regs;
	regs.ax = 0x4900;	/* free block */
	regs.es = base;		/* base paragraph */
#ifdef AZTEC86
	if (sysint21(&regs,&regs) & 1)
#endif
#ifdef MSC
	sysint21(&regs,&regs);
	if (regs.cflag)
#endif
		return regs.ax;
	return 0;
}

/*
** dread - read from file or device to a particular paragraph in memory.
** allows small model use of buffers outside the current data space
*/
unsigned dread(handle,buf,count)
	int handle; paragraph buf; unsigned count;
{
	struct registers regs;
	regs.ax = 0x3f00;	/* read dos func */
	regs.bx = handle;
	regs.ds = buf;		/* segment of buffer */
	regs.dx = 0;		/* offset of buffer */
	regs.cx = count;
#ifdef AZTEC86
	if (sysint(0x21,&regs,&regs) & 1)
#endif
#ifdef MSC
	sysint21(&regs,&regs);
	if (regs.cflag)
#endif
	{
		errno = (int)regs.ax;
		return ERROR;
	}
	return (int)regs.ax;
}

/*
** dwrite - write from file or device to a particular paragraph in memory.
** allows small model use of buffers outside the current data space
*/
unsigned dwrite(handle,buf,count)
	int handle; paragraph buf; unsigned count;
{
	struct registers regs;
	regs.ax = 0x4000;
	regs.bx = handle;
	regs.ds = buf;	/* segment of buffer */
	regs.dx = 0;	/* offset of buffer */
	regs.cx = count;
#ifdef AZTEC86
	if (sysint(0x21,&regs,&regs) & 1)
#endif
#ifdef MSC
	sysint21(&regs,&regs);
	if (regs.cflag)
#endif
	{
		errno = regs.ax;
		return ERROR;
	}
	return (int)regs.ax;
}

#define BUFSIZE ((unsigned)63*1024)
int filecopy(target,source)
char *target,*source;
{
	void (*oldsig)();
	paragraph buffer; long ftime();
    int t,s; int rval = 0;
	unsigned avail,r;

	oldsig = signal(SIGINT,SIG_IGN);

	/*
	** get a 63K buffer, or as much as is available
	*/
	if (0 == (buffer = get_big_mem(BUFSIZE,&avail)))
	{
		/*
		** if we get zero back, then avail contains the number
		** of bytes currently available
		*/
		if (0 == (buffer = get_big_mem(avail,&r)))
		{
			fprintf(stderr,"no memory available\n");
			exit(ERROR);
		}
	}
	else 
		avail = BUFSIZE;	/* we got all we asked for */
    if (-1 == (s = open(source,O_RDONLY)))
    {
		fprintf(stderr,"%s : can't open %s\n",me,source);
		rval = -1;
		goto mungeout;
    }
#ifdef AZTEC86
    if (-1 == (t = open(target,O_TRUNC|O_CREAT|O_WRONLY)))
#endif
#ifdef MSC
    if (-1 == (t = open(target,O_TRUNC|O_CREAT|O_WRONLY,S_IREAD|S_IWRITE)))
#endif
    {
		fprintf(stderr,"%s : can't open %s\n",me,target);
		rval = -1;
		goto mungeout;
    }
    while(0 != (r = dread(s,buffer,avail)) && r != -1)
    {
		if(-1 == dwrite(t,buffer,r))
		{
		    fprintf(stderr,"%s : error writing %s\n",me,target);
		    rval = -1;
			goto mungeout;
		}
    }
	/* update the file time of target to match source */
	if (-1L == (ftime(1,t,ftime(0,s,0L))))
	{
		perror("cp");
		rval = -1;
	}
mungeout:
	close(t); 
    close(s);
	(void)free_big_mem(buffer);	/* get rid of external buffer */
	(void)signal(SIGINT,oldsig);
	return (rval);
}

#include <errno.h>
typedef struct
{
    char dos_reserved[21];
    char attribute;
    unsigned file_time;
    unsigned file_date;
    long file_size;
    char file_name[13];
} 
fcb;
fcb dir;
/*lint -e529 */
int dirp(s)
char *s;
{
	register int junk1,junk2;
	/* handle all of the stupid special cases */
	if (0 == strcmp(&s[1],":") || /* root directory on a drive */
		0 == strcmp(&s[0],"/") ||
		0 == strcmp(&s[0],"\\") || /* root directory default drive */
		0 == strcmp(&s[1],":/") || /* root directory another drive */
		0 == strcmp(&s[1],":\\") ||
		0 == strcmp(&s[1],":.") ||
		0 == strcmp(&s[0],"."))
	{
		return 1;
	}
	if (0 == strcmp(s,".."))			/* parent of this directory */
	{
		int returnval;
		char *current,*getcwd();
		current = getcwd((char *)NULL,64);		/* get current directory */
		/* everything except root has a parent */
		returnval = strlen(current) != 0;
		free(current);
		return returnval;
	}
    /* set the disk transfer address */
    bdos(0x1A,&dir);
    /* do a search first for the directory path */
    return (bdos(0x4E,s,0x10) == 0 && bdos(0x4E,s,0) != 0);
}
/*lint +e529 */	
int filep(s)
char *s;
{
    /* set the disk transfer address */
    bdos(0x1A,&dir);
    /* do a search first for the directory path */
    return bdos(0x4E,s,0) == 0;
}
char *fname_part(s)
register char *s;
{
    register char *r; 
    char *strrchr();
    if ((r = strrchr(s,'/')) || (r = strrchr(s,'\\')))
	{
		return r+1;
	}
    if (r = strrchr(s,':'))
	{
		return r+1;
	}
    return s;
}
char *path_part(s)
register char *s;
{
	static char buffer[64];
    register char *r; 
    char *strrchr();
	strcpy(buffer,s);	/* copy string */
	if ((r = strrchr(buffer,'/')) ||
		(r = strrchr(buffer,'\\')))
	{
		*++r = '\0';
		for (r = buffer; '\0' != *r; r++)
			*r = (*r == '\\' ? '/' : *r);
		return buffer;
	}

	if (r = strrchr(buffer,':'))
	{
		*++r = '\0';
		return buffer;
	}
	return NULL;
}
char *drive_part(s)
register char *s;
{
	static char buffer[64];
    char *strrchr();
	strcpy(buffer,s);	/* copy string */
	if (buffer[1] == ':')
	{
		buffer[2] = '\0';
		return buffer;
	}
	return NULL;
}
/*lint -restore */
